package com.dsic.gj_erp.config;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zaxxer.hikari.HikariDataSource;
import org.hibernate.boot.model.naming.Identifier;
import org.hibernate.boot.model.naming.PhysicalNamingStrategy;
import org.hibernate.boot.model.naming.PhysicalNamingStrategyStandardImpl;
import org.hibernate.engine.jdbc.env.spi.JdbcEnvironment;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.datasource.init.DataSourceInitializer;
import org.springframework.jdbc.datasource.init.ResourceDatabasePopulator;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;
import java.util.Properties;

@Configuration
@EnableJpaRepositories(
        basePackages = "com.dsic.gj_erp",
        entityManagerFactoryRef = "manEntityManagerFactory",
        transactionManagerRef = "manTransactionManager"
)
public class JapConfig {
    @Autowired
    private Environment env; //1

    @Bean
    @ConfigurationProperties(prefix = "spring.datasource.master")
    public DataSourceProperties japDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean("jpaDataSource")
    public DataSource jpaDataSource() {
        DataSourceProperties manDataSourceProperties = japDataSourceProperties();
        return DataSourceBuilder.create()
                .driverClassName(manDataSourceProperties.getDriverClassName())
                .url(manDataSourceProperties.getUrl())
                .username(manDataSourceProperties.getUsername())
                .password(manDataSourceProperties.getPassword())
                .build();
    }

    @Bean
    public PlatformTransactionManager manTransactionManager() { //4
        EntityManagerFactory factory = manEntityManagerFactory().getObject();
        JpaTransactionManager jpaTransactionManager = new JpaTransactionManager(factory);
        System.out.println(jpaTransactionManager.isNestedTransactionAllowed());
        return jpaTransactionManager;
    }

    @Bean
    public PhysicalNamingStrategy physicalNamingStrategy() {
        return new PhysicalNamingStrategyStandardImpl() {
            @Override
            public Identifier toPhysicalTableName(Identifier name, JdbcEnvironment jdbcEnvironment) {
                return Identifier.toIdentifier(StringUtils.camelToUnderline(name.getText()));
            }
        };
    }

    @Bean
    public LocalContainerEntityManagerFactoryBean manEntityManagerFactory() {
        LocalContainerEntityManagerFactoryBean factory =
                new LocalContainerEntityManagerFactoryBean();
        factory.setDataSource(jpaDataSource());
        factory.setPackagesToScan("com.dsic.gj_erp");
        factory.setJpaVendorAdapter(new HibernateJpaVendorAdapter());
        Properties jpaProperties = new Properties();
        //设置控制台打印sql
        jpaProperties.put("hibernate.show_sql", env.getProperty("hibernate.show_sql"));
        //字段驼峰转下划线
        jpaProperties.put("hibernate.physical_naming_strategy", env.getProperty("hibernate.naming.physical_strategy"));
        jpaProperties.put("hibernate.dialect",env.getProperty("hibernate.dialect"));
        factory.setJpaProperties(jpaProperties);
        return factory;
    }

    @Bean
    public DataSourceInitializer manDataSourceInitializer() {//6
        DataSourceInitializer dsInitializer = new DataSourceInitializer();
        HikariDataSource dataSource = (HikariDataSource) jpaDataSource();
        dataSource.setMaximumPoolSize(9999);
        dataSource.setLeakDetectionThreshold(2000);
        dsInitializer.setDataSource(dataSource);
        ResourceDatabasePopulator dbPopulator = new ResourceDatabasePopulator();
        dsInitializer.setDatabasePopulator(dbPopulator);
        dsInitializer.setEnabled(env.getProperty("spring.datasource.master.initialize",
                Boolean.class, false));
        return dsInitializer;
    }
}
